package com.dragon.tj.portal.auth.config;

import com.dragon.tj.portal.auth.service.JwtTokenAuthenticationFilter;
import com.dragon.tj.portal.auth.service.MyCasAuthenticationEntryPoint;
import com.dragon.tj.portal.auth.service.MySimpleUrlAuthenticationSuccessHandler;
import com.dragon.tj.portal.auth.service.MyUserDetailsByNameServiceWrapper;
import com.dragon.tj.portal.auth.service.MyUserDetailsService;
import org.jasig.cas.client.validation.Cas20ServiceTicketValidator;
import org.jasig.cas.client.validation.TicketValidator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.web.servlet.context.ServletWebServerApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.authentication.ProviderManager;
import org.springframework.security.cas.ServiceProperties;
import org.springframework.security.cas.authentication.CasAuthenticationProvider;
import org.springframework.security.cas.web.CasAuthenticationFilter;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityCustomizer;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.web.AuthenticationEntryPoint;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.security.web.authentication.SimpleUrlAuthenticationFailureHandler;

@Configuration
@EnableWebSecurity
public class WebSecurityConfig {
    @Value("${cas.base.url}")
    private String casBaseUrl;

    @Value("${cas.login.url}")
    private String casLoginUrl;

    @Value("${cas.service.url}")
    private String casServiceUrl;

    @Value("${cas.filter.url}")
    private String casFilterUrl;

    @Value("${cas.target.url}")
    private String casTargetUrl;

    @Value("${cas.failure.url}")
    private String casFailureUrl;

    @Autowired
    private ServletWebServerApplicationContext context;

    @Autowired
    private JwtTokenAuthenticationFilter jwtTokenAuthenticationFilter;
    @Autowired
    private MyUserDetailsService userDetailsService;
    @Autowired
    private MySimpleUrlAuthenticationSuccessHandler mySimpleUrlAuthenticationSuccessHandler;

    private static StringBuilder whiteList = new StringBuilder();
    private static String whiteListSplit = ",";

    /**
     *
     *白名单
     */
    static {
        whiteList.append("/test/login" + whiteListSplit);
        whiteList.append("/subscribe/*" + whiteListSplit);
        whiteList.append("/test/test1" + whiteListSplit);
        whiteList.append("/file/**" + whiteListSplit);
    }

    @Bean
    public WebSecurityCustomizer webSecurityCustomizer() {

        return (web) -> web.ignoring()
                // 认证成功后才会忽略
                .antMatchers("/resources/**");
    }


    @Bean
    public SecurityFilterChain securityFilterChain(HttpSecurity http) throws Exception {
        http
                // CSRF禁用，因为不使用session
                .csrf().disable().cors()
                .and()
                .authorizeRequests()
                .antMatchers(whiteList.toString().split(whiteListSplit)).permitAll()
                .anyRequest().authenticated()
                .and()
                .sessionManagement().sessionCreationPolicy(SessionCreationPolicy.STATELESS)
                .and()
                .exceptionHandling(exceptions -> exceptions.authenticationEntryPoint(authenticationEntryPoint()))
                .addFilter(casAuthenticationFilter())
                .addFilterBefore(jwtTokenAuthenticationFilter, CasAuthenticationFilter.class)
                // .addFilterBefore(casLogoutFilter(), LogoutFilter.class)
                // .addFilterBefore(singleSignOutFilter(), CasAuthenticationFilter.class);
                // .addFilterBefore(new SingleSignOutFilter(), CasAuthenticationFilter.class)
                .logout();

        return http.build();
    }

    public AuthenticationEntryPoint authenticationEntryPoint() {
        MyCasAuthenticationEntryPoint casAuthenticationEntryPoint = new MyCasAuthenticationEntryPoint();
        casAuthenticationEntryPoint.setLoginUrl(this.casLoginUrl);
        casAuthenticationEntryPoint.setServiceProperties(serviceProperties());
        return casAuthenticationEntryPoint;
    }

    public CasAuthenticationFilter casAuthenticationFilter() {
        CasAuthenticationFilter filter = new CasAuthenticationFilter();
        filter.setFilterProcessesUrl(casFilterUrl);
        filter.setAuthenticationFailureHandler(new SimpleUrlAuthenticationFailureHandler(casFailureUrl));

        CasAuthenticationProvider casAuthenticationProvider = casAuthenticationProvider(userDetailsService);
        filter.setAuthenticationManager(new ProviderManager(casAuthenticationProvider));

        mySimpleUrlAuthenticationSuccessHandler.setDefaultTargetUrl(casTargetUrl);
        filter.setAuthenticationSuccessHandler(mySimpleUrlAuthenticationSuccessHandler);

        return filter;
    }

    public CasAuthenticationProvider casAuthenticationProvider(UserDetailsService userDetailsService) {
        CasAuthenticationProvider provider = new CasAuthenticationProvider();
        provider.setAuthenticationUserDetailsService(new MyUserDetailsByNameServiceWrapper<>(userDetailsService));
        provider.setServiceProperties(serviceProperties());
        provider.setTicketValidator(ticketValidator());
        provider.setKey("key");
        return provider;
    }

    public ServiceProperties serviceProperties() {
        ServiceProperties serviceProperties = new ServiceProperties();
        serviceProperties.setService(casServiceUrl);
        return serviceProperties;
    }

    private TicketValidator ticketValidator() {
        return new Cas20ServiceTicketValidator(this.casBaseUrl);
    }

}
